#------------
# edbind.tcl
#------------
#
#  Add more functions to the text widget.
#
#--
#  Initialize global variables.
#--
set moved 0
set comeback "0"
set buffer ""
set lineno 1
#--
#  Move the cursor to a given index.
# 	t	: the path of the text widget containing the cursor
#	index	: the destination
#	flag	: the flag which flags whether the range of selection
#		: should be changed or not.
#--
proc SetCursor {t index flag} {
  global moved lineno
  #--
  #  Change the selection range.
  #--
  if $flag {
    #--
    #  If none selected, set the anchor at the cursor.
    #--
    if {[$t tag nextrange sel 1.0 end] == ""} {
      $t mark set anchor insert
    } 
    #--
    #  Identify the selection range.
    #--
    if [$t compare $index < anchor] {
      set first $index
      set last anchor
    } else {
      set first anchor
      set last $index
    }
    #--
    #  Clear the selection above.
    #--
    $t tag remove sel 1.0 $first
    #--
    #  Register the new selection.
    #--
    $t tag add sel $first $last
    #--
    #  Clear the selection below.
    #--
    $t tag remove sel $last end
  #--
  #  Clear the selection.
  #--
  } else {
    $t tag remove sel 1.0 end
  }
  #--
  #  Move the cursor.
  #--
  $t mark set insert $index
  set lineno [GetLineNo $t]
  #--
  #  Adjust the view.
  #--
  $t see insert
  # $t yview -pickplace insert
  #--
  #  Set the global variables required by GetLineIndex.
  #--
  if {$moved == -1} {
    set moved 0
  } elseif {$moved == 0} {
    set moved 1
  }
}
#--
#  Get the index of the destination.
# 	t	: the widget path containing the cursor.
# 	n	: the relative line number of the destination.
#--
proc GetLineIndex {t n} {
  global moved comeback
  #--
  #  Get the current cursor index.
  #--
  set i [$t index insert]
  scan $i "%d.%d" line char
  #--
  #  Get the line number of the destination.
  #--
  set line [expr $line + $n]
  #--
  #  When the cursor moved beyound the scope of the text widget.
  #--
  if $moved {
     set comeback $char
  }
  #--
  #  Set the index of the destination.
  #--
  set new [$t index $line.$comeback]
  #--
  #  If the index is outside the text.
  #--
  if {[$t compare $new > end] || [$t compare $new < 1.0]} {
    set new $i
  }
  if {[$t compare $new > "$line.0 lineend"]} {
    set new [$t index "$line.0 lineend"]
  }
  #--
  #  Reset the flag.
  #--
  set moved -1
  return $new
}
#--
#  Copy the selected to a buffer.
# 	t	: the text widget path.
#--
proc CopyString t {
  global buffer
  set indeces [$t tag nextrange sel 1.0]
  if {$indeces == ""} return
  set buffer [eval $t get $indeces]
}
#--
#  Cut the selected to the buffer.
# 	t	: the text widget path.
#--
proc CutString t {
  global buffer
  set indeces [$t tag nextrange sel 1.0]
  if {$indeces == ""} {
    return
  }
  set buffer [eval $t get $indeces]
  eval $t delete $indeces
}
#--
#  Paste the buffer and clear the selection.
# 	t	: the text widget path.
#--
proc PasteString t {
  global buffer
  $t insert insert $buffer
}
#--
#  More and more functions.
# 	t	: the text widget path.
#
#  (Update Record)
#	96/04/27  K.Fujii	Modified for use with a 5300ce.
#				Those special keys which are only available
#				for desk-top Mac's are not fixed.
#--
proc PowerUpText t {
  global tw lineno
  #--
  #  Activate the DELETE key.
  #  96/04/27  K.Fujii	Don't do this, since my 5300ce doesn't have
  #			any <Backspace>.
  #--
#  bind $t <Delete> {
#    %W delete insert
#    # %W yview -pickplace insert
#    %W see insert
#  }
#  bind $t <Delete> {
#    if {[%W tag nextrange sel 1.0 end] != ""} {
#        %W delete sel.first sel.last
#        set lineno [IndexToLineNo [%W index sel.first]]
#    } elseif [%W compare insert != 1.0] {
#        %W delete insert-1c
#	set lineno [IndexToLineNo [%W index {insert -1c}]]
#        %W see insert
#    }
#  }
  #--
  #  <ButtonPress-1>.
  #  96/04/27  K.Fujii	Update the line number
  #--
  bind $t <ButtonPress-1> [bind Text <ButtonPress-1>]
  bind $t <ButtonPress-1> {+set moved 1}
  bind $t <ButtonPress-1> {+set lineno [GetLineNo %W]}
  bind $t <Shift-1> [bind Text <Shift-1>]
  #--
  #  Special keys.
  #--
  bind $t <Left> {
    SetCursor %W [%W index {insert -1c}] 0
  }
  bind $t <Right> {
    SetCursor %W [%W index {insert + 1c}] 0
  }
  bind $tw <Up> {
    SetCursor %W [GetLineIndex %W -1] 0
  }
  bind $tw <Down> {
    SetCursor %W [GetLineIndex %W 1] 0
  }
  bind $tw <Home> {
    SetCursor %W [%W index {insert linestart}] 0
  }
  bind $tw <End> {
    SetCursor %W [%W index {insert lineend}] 0
  }
  bind $tw <Control-Home> {
    SetCursor %W 1.0 0
  }
  bind $tw <Control-End> {
    SetCursor %W end 0
  }
  #--
  #  Combined keys.
  #--
  bind $t <Shift-Left> {
    SetCursor %W [%W index {insert - 1c}] 1
  }
  bind $t <Shift-Right> {
    SetCursor %W [%W index {insert + 1c}] 1
  }
  bind $tw <Shift-Up> {
    SetCursor %W [GetLineIndex %W -1] 1
  }
  bind $tw <Shift-Down> {
    SetCursor %W [GetLineIndex %W 1] 1
  }
  bind $tw <Shift-Home> {
    SetCursor %W [%W index {insert linestart}] 1
  }
  bind $tw <Shift-End> {
    SetCursor %W [%W index {insert lineend}] 1
  }
  bind $tw <Shift-Control-Home> {
    SetCursor %W 1.0 1
  }
  bind $tw <Shift-Control-End> {
    SetCursor %W end 1
  }
  #--
  #  96/04/27  K.Fujii	Keep the default emacs-like key bindings 
  #			and update the line number accordingly.
  #--
  bind $t <Control-h> {+set lineno [IndexToLineNo [%W index {insert -1c}]]}
  bind $t <Control-b> {+set lineno [IndexToLineNo [%W index {insert -1c}]]}
  bind $t <Control-f> {+set lineno [IndexToLineNo [%W index {insert +1c}]]}
  bind $t <Control-p> {+set lineno [IndexToLineNo [GetLineIndex %W -1]]}
  bind $t <Control-n> {+set lineno [IndexToLineNo [GetLineIndex %W +1]]}
  bind $t <Meta-b> \
	{+set lineno [IndexToLineNo [%W index {insert -1c wordstart}]]}
  bind $t <Meta-f> {+set lineno [IndexToLineNo [%W index {insert wordend}]]}
  bind $t <Meta-less> {+set lineno 1}
  bind $t <Meta-greater> {+set lineno [IndexToLineNo [%W index end]]}
  bind $t <Return> {+set lineno [IndexToLineNo [GetLineIndex %W +1]]}
  #--
  #  Copy/Cut/Paste.
  #--
#  bind $tw <Control-c> {
#    CopyString %W
#  }
#  bind $tw <Control-x> {
#    CutString %W
#  }
#  bind $tw <Control-v> {
#    PasteString %W
#  }
}

#--
#  96/04/27  K.Fujii	Use <Delete> as <Backspace>.
#--
bind Entry <Delete> {
  tkEntryBackspace %W
}


proc IndexToLineNo index {
  scan $index %d.%d line char
  return $line
}