#--------------
# edfileop.tcl
#--------------
#
#  File handling.
#
#--
#  Initialize global variables.
#--
set fileName ""
set contents ""
#--
#  Clear the text and reinitialize "fileName" and "contents".
# 	t	: the text widget path.
#--
proc CreateNew t {
  global fileName contents tw
  #--
  #  If something there in the text widget, try to save it before
  #  clearing the text.
  #--  
  if {[string trimright $contents] != [string trimright [$t get 0.0 end]]} {
    set result [tk_dialog .d dialog "No file name is given yet. 
Do you want to set it?" warning 0 "Yes" "No" "Cancel"]
    switch $result {
      0 {
        if {$fileName == ""} {
          if ![SaveAs $t] {return 0}
        } else {
          if ![SaveFile $t] {return 0}
        }
      } 
      2 {return 0}
    }
  }
  #--
  #  Clear the global variables.
  #--
  set fileName ""
  set contents ""
  focus $t
  $t delete 0.0 end
  return 1
}

#--
#  Overwrite the text to "fileName" and update "contents".
# 	t	: the text widget path.
#--
proc SaveFile t {
  global fileName contents tw
  #--
  #  If this is a new text, try "SaveAs".
  #--
  if {$fileName == ""} {
    if [SaveAs $t] {
      return 1
    } else {
      return 0
    }
  }
  #--
  #  If the file is writable, save the text.
  #--
  if ![file writable $fileName] {
    tk_dialog .d dialog "Can't save the text. Read only!" warning 0 "OK"
    return 0
  }
  set contents [$t get 0.0 end]
  set fd [open $fileName w]
  puts $fd $contents 
  close $fd
  return 1
}


#--
#  Save the text as "fileName".
# 	t	: the text widget path.
#--
proc SaveAs t {
  global fileName contents tw
  #--
  #  Get the file name.
  #--
  if {[set file [SelectFile .selfile "Save File" 1]] == ""} {
    tk_dialog .d dialog "Give me a file name" warning 0 "OK"
    return 0
  }
  #--
  #  If the file is already there.
  #--
  if [file exists $file] {
    set result [tk_dialog .d dialog "The file already exists.
Overwrite it?" warning 0 "Yes" "No"]
    if {$result == 1} {return 0}
  }
  #--
  #  If the file is read-only, do nothing and return.
  #--
  if {[file exist $file] && ![file writable $file]} {
    tk_dialog .d dialog "Can't save the text. Read only!" warning 0 "OK"
    return 0
  }
  #--
  #  Use absolute path.
  #--
  if ![kstring match {[/~]*} $file] {
    if {[pwd] == "/"} {
      set file /$file
    } else {
      set file [pwd]/$file
    }
  }
  #--
  #  Update "fileName" and "contents" and save "contents" to "fileName".
  #--
  set fileName $file
  set contents [$t get 0.0 end]
  set fd [open $file w]
  puts $fd $contents 
  close $fd
  return 1
}

#--
#  Read a file to the text and update "fileName" and "contents".
# 	t	: the text widget path.
#--
proc ReadFile t {
  global fileName contents tw lineno
  #--
  #  If the text has been modified.
  #--
  if {[string trimright $contents] != [string trimright [$t get 0.0 end]]} {
    set result [tk_dialog .d dialog "The text has been modified. 
Do you want to save it?" warning 0 "Yes" "No" "Cancel"]
    switch $result {
      0 {
        if {$fileName == ""} {
          if ![SaveAs $t] {return 0}
        } else {
          if ![SaveFile $t] {return 0}
        }
      } 
      2 {return 0}
    }
  }
  #--
  #  Get the file name.
  #--
  if {[set file [SelectFile .selfile "Read File" 0]] == ""} {
    tk_dialog .d dialog "Give me a file name." warning 0 "OK"
    return 0
  }
  #--
  #  If the file is read-protected, do nothing and return.
  #--
  if ![file readable $file] {
    tk_dialog .d dialog "The file is not readable." warning 0 "OK"
    return 0
  }
  #--
  #  Use absolute path.
  #--
  if ![kstring match {[/~]*} $file] {
    if {[pwd] == "/"} {
      set file /$file
    } else {
      set file [pwd]/$file
    }
  }
  #--
  #  Update "fileName" and "contents" and read "fileName" to the text.
  #--
  set fileName $file
  set fd [open $file r]
  set contents [read $fd]
  close $fd
  $t delete 0.0 end
  $t insert 0.0 $contents
  $t mark set insert 1.0
  set lineno 1
  focus $t
  return 1
}

#--
#  Open "file" and update "fileName" and "contents".
# 	t	: the text widget path.
#--
proc OpenFile {t file} {
  global fileName contents tw lineno
  #--
  #  If the text has been modified.
  #--
  if {[string trimright $contents] != [string trimright [$t get 0.0 end]]} {
    set result [tk_dialog .d dialog "The text has been modified. 
Do you want to save it?" warning 0 "Yes" "No" "Cancel"]
    switch $result {
      0 {
        if {$fileName == ""} {
          if ![SaveAs $t] {return 0}
        } else {
          if ![SaveFile $t] {return 0}
        }
      } 
      2 {return 0}
    }
  }
  #--
  #  Get the file name.
  #--
  if {$file == ""} {
    tk_dialog .d dialog "Give me a file name." warning 0 "OK"
    return 0
  }
  #--
  #  If the file is read-protected, do nothing and return.
  #--
  if ![file readable $file] {
    tk_dialog .d dialog "The file is not readable." warning 0 "OK"
    return 0
  }
  #--
  #  Use absolute path.
  #--
  if ![kstring match {[/~]*} $file] {
    if {[pwd] == "/"} {
      set file /$file
    } else {
      set file [pwd]/$file
    }
  }
  #--
  #  Update "fileName" and "contents" and read "fileName" to the text.
  #--
  set fileName $file
  set fd [open $file r]
  set contents [read $fd]
  close $fd
  $t delete 0.0 end
  $t insert 0.0 $contents
  $t mark set insert 1.0
  set lineno 1
  focus $t
  return 1
}

#--
#  Insert a file to the cursor position.
# 	t	: the text widget path.
#--
proc InsertFile t {
  global tw
  #--
  #  Get the file name.
  #--
  if {[set file [SelectFile .selfile "Insert File" 0]] == ""} {
    tk_dialog .d dialog "Give me a file name." warning 0 "OK"
    return 0
  }
  #--
  #  If the file is read-protected, do nothing and return.
  #--
  if ![file readable $file] {
    tk_dialog .d dialog "The file is not readable." warning 0 "OK"
    return 0
  }
  #--
  #  Insert the file into the text.
  #--
  set fd [open $file r]
  $t insert insert [read $fd]
  close $fd
  return 1
}

#--
#  Exit the application.
# 	t	: the text widget path.
#--
proc ExitApp t {
  global fileName contents tw
  #--
  #  If the text is modified, try to save it.
  #--
  if {[string trimright $contents] != [string trimright [$t get 0.0 end]]} {
    set result [tk_dialog .d dialog "The text has been modified. 
Do you want to save it?" warning 0 "Yes" "No" "Cancel"]
    switch $result {
      0 {
        if {$fileName == ""} {
          if ![SaveAs $t] {return 0}
        } else {
          if ![SaveFile $t] {return 0}
        }
      } 
      2 {return 0}
    }
  }
  #--
  #  That's it folks.
  #--
  exit 0
}

#--
#  Change the title of the toplevel window.
# 	t	: the text widget path.
#	name1	: dummy.
#	name2	: dummy.
#	op	: dummy.
#--
proc ChangeTitle {t name1 name2 op} {
  global fileName tw
  wm title [winfo toplevel $t] "Tkpad - $fileName"  
}

#--
#  Update the "title" everytime "fileName" changed.
#--
trace variable fileName w {ChangeTitle $tw}
