#-------------
# edserch.tcl
#-------------
#
#  Search and Replace.
#
#--
#  Initialize global variables.
#--
set direction "down"
set searchCmd "searchDown"
set replaceCmd "replaceDown"
set selRange ""
#--
#  Search down from the cursor position.
# 	t	: top level widget path.
# 	search	: target string.
# 	flag	: flag to allow case-sensitive.
#--
proc searchDown {t search flag} {
  global selRange moved lineno
  #--
  #  Get the current cursor position and the total number of text lines.
  #--
  scan [$t index insert] %d.%d line char
  scan [$t index end] %d numLines    
  #--
  #  If flag = 0, carry out case-insensitive search.
  #--
  if !$flag {set search [kstring tolower $search]}
  #--
  #  Clear the selection.
  #--
  $t tag remove sel 0.0 end
  #--
  #  Now start the search.
  #--
  for {set i $line} {$i <= $numLines} {incr i} {
    if {$i == $line} {
      set string [$t get $i.$char "$i.0 lineend"]
    } else {
      set string [$t get $i.0 "$i.0 lineend"]
    }
    if !$flag {set string [kstring tolower $string]}
    if {[set index [kstring first $search $string]] == -1} {
      continue
    }
    if {$i == $line} {incr index $char}
    set end [expr $index + [kstring length $search]]
    set selRange "$i.$index $i.$end"
    eval $t tag add sel $selRange
    $t mark set insert $i.$end
    # $t yview -pickplace insert
    set lineno [GetLineNo $t]
    $t see insert
    set moved 1 
    return
  }
  tk_dialog .d Search "No match" {} 0 "OK"
}

#--
#  Search up from the cursor position.
# 	t	: top level widget path.
# 	search	: target string.
# 	flag	: flag to allow case-sensitive.
#--
proc searchUp {t search flag} {
  global selRange moved lineno
  #--
  #  Get the current cursor position.
  #--
  set range [$t tag nextrange sel 1.0]
  scan [$t index insert] %d.%d line char
  #--
  #  Adjust the search range.
  #--
  if {$range != "" && $range == $selRange} {
    if [$t compare [lindex $range 1] == $line.$char] {
      scan $range %d.%d line char
    }
  }
  #-- 
  #  If flag = 0, carry out case-insensitive search.
  #--
  if !$flag {set search [kstring tolower $search]}
  #--
  #  Now start the search.
  #--
  $t tag remove sel 0.0 end
  for {set i $line} {$i > 0} {incr i -1} {
    if {$i == $line} {
      set string [$t get $i.0 $i.$char]
    } else {
      set string [$t get $i.0 "$i.0 lineend"]
    }
    if !$flag {set string [kstring tolower $string]}
    if {[set index [kstring last $search $string]] == -1} {
      continue
    }
    set end [expr $index + [kstring length $search]]
    set selRange "$i.$index $i.$end"
    eval $t tag add sel $selRange
    $t mark set insert $i.$end
    # $t yview -pickplace insert
    set lineno [GetLineNo $t]
    $t see insert
    set moved 1
    return
  }
  tk_dialog .d Search "No match" warning 0 "OK"
}

#--
#  Replace the matched string.
# 	t	: top level widget path.
# 	replace	: new string.
#--
proc replaceString {t replace} {
  global selRange
  #--
  #  Check consistency.
  #--
  set range [$t tag nextrange sel 1.0]
  if {$range == "" || $range != $selRange} return
  if ![$t compare [lindex $selRange 1] == insert] return
  #--
  #  Now replace the string.
  #--
  set first [lindex $selRange 0]
  set last [lindex $selRange 1]
  $t delete $first $last
  $t insert $first $replace
  set last [$t index "$first + [kstring length $replace] c"]
  $t tag add sel $first $last
  set selRange "$first $last"
}

#--
#  Replace all the matched string below.
# 	t	: top level widget path.
# 	search	: target string.
#	replace : new string.
# 	flag	: flag to allow case-sensitive.
#--
proc replaceDown {t search replace flag} {
  #--
  #  Get the current position and the total number of the text lines.
  #--
  scan [$t index insert] %d.%d line char 
  scan [$t index end] %d numLines    
  #--
  #  Allow case-insensitive search, if flag = 0.
  #--
  if !$flag {
    set search [kstring tolower $search]
  }
  #--
  #  Get the string lengths.
  #--
  set ls [kstring length $search]
  set lr [kstring length $replace]
  #--
  #  Now start the replacement.
  #--
  set cnt 0
  for {set i $line} {$i <= $numLines} {incr i} {
    if {$i == $line} {
      set string [$t get $i.$char "$i.0 lineend"]
    } else {
      set string [$t get $i.0 "$i.0 lineend"]
    }
    if !$flag {
      set string [kstring tolower $string]
    }
    if {[set index [kstring first $search $string]] == -1} {
      continue
    }
    if {$i == $line} {incr index $char}
    while 1 {
      $t delete $i.[expr $index] $i.[expr $index+$ls]
      $t insert $i.[expr $index] $replace
      incr cnt
      set offset [expr $index +	$lr]
      set string [$t get $i.$offset "$i.0 lineend"]
      if !$flag {
        set string [kstring tolower $string]
      }
      if {[set index [kstring first $search $string]]  == -1} {
        break
      }
      incr index $offset 
    }
  }
  #--
  #  Report the number of replacements.
  #--
  tk_dialog .d Replace [format "Replaced %d strings" $cnt] warning 0 "OK"
}

#--
#  Replace all the matched string above.
# 	t	: top level widget path.
# 	search	: target string.
#	replace : new string.
# 	flag	: flag to allow case-sensitive.
#--
proc replaceUp {t search replace flag} {
  global selRange
  #--
  #  Get the current position and the total number of the text lines.
  #--
  set range [$t tag nextrange sel 1.0]
  if {$range != "" && $range == $selRange} {
    scan [lindex $range 0] %d.%d line char
  } else {
    scan [$t index insert] %d.%d line char
  }
  #--
  #  Allow case-insensitive search, if flag = 0.
  #--
  if !$flag {
    set search [kstring tolower $search]
  }
  #--
  #  Get the string lengths.
  #--
  set ls [kstring length $search]
  set lr [kstring length $replace]
  #--
  #  Now start the replacement.
  #--
  set cnt 0
  for {set i $line} {$i > 0} {incr i -1} {
    if {$i == $line} {
      set string [$t get $i.0 $i.$char]
    } else {
      set string [$t get $i.0 "$i.0 lineend"]
    }
    if !$flag {
      set string [kstring tolower $string]
    }
    if {[set index [kstring last $search $string]] == -1} {
      continue
    }
    while 1 {
      $t delete $i.$index $i.[expr $index+$ls]
      $t insert $i.$index $replace
      incr cnt
      set string [$t get $i.0 $i.$index]
      if !$flag {
        set string [kstring tolower $string]
      }
      if {[set index [kstring last $search $string]]  == -1} {
        break
      }
    }
  }
  #--
  #  Report the number of replacements.
  #--
  tk_dialog .d Replace [format "Replaced %d strings" $cnt] warning 0 "OK"
}

#--
#  Search/Replace Window.
# 	w	: top level widget path.
#	title	: title.
# 	search	: target string.
# 	flag	: flag to allow replace.
#--
proc SearchString {w title t flag} {
  global selRange searchString 
  #--
  #  Setup widgets.
  #--
  toplevel $w
  wm title $w $title
  frame $w.require
  frame $w.require.search
  label $w.require.search.label -width 16
  entry $w.require.search.entry -width 20 -bd 2 -relief sunken \
      -textvariable searchString
  checkbutton $w.require.case -relief flat \
      -text "Case Sensitive" -variable check
  radiobutton $w.require.up -relief flat \
      -text "Search Up" -variable direction -value "up"
  radiobutton $w.require.down -relief flat \
      -text "Search Down" -variable direction -value "down"
  button $w.next -width 14 -text "Find Next" -command \
      "\$searchCmd $t \$searchString \$check" 
  button $w.quit -width 14 -text "Cancel" -command "destroy $w"
  if $flag {
    frame $w.require.replace 
    label $w.require.replace.label -width 16 -text "Replace with"
    entry $w.require.replace.entry -width 20 -bd 2 -relief sunken \
        -textvariable replaceString
    button $w.replace -width 14 -text "Replace" -command \
	"replaceString $t  \$replaceString; \$searchCmd $t \$searchString \$check"
    button $w.all -width 14 -text "Replace All" -command \
        "\$replaceCmd $t \$searchString \$replaceString \$check"
  }
  #--
  #  Pack the widgets.
  #--
  pack $w.require.search.label $w.require.search.entry -side left
  pack $w.require.search $w.require.case $w.require.up $w.require.down \
      -anchor w -padx 5 -pady 5
  pack $w.require -side left -anchor n -padx 5 -pady 5
  pack $w.next $w.quit -padx 5 -pady 5
  if $flag {
    pack $w.require.replace.label $w.require.replace.entry -side left
    pack $w.require.replace -after $w.require.search -padx 5 -pady 5
    pack $w.replace $w.all -after $w.next -padx 5 -pady 5
  }
  #--
  #    
  #--
  if $flag {
    $w.require.search.label configure -text "Find"
  } else {
    $w.require.search.label configure -text "Find"
  } 
  #--
  #  Handle <Return> in entry boxes.
  #--
  bind $w.require.search.entry <Any-Return> "$w.next invoke"
  if $flag {
    bind $w.require.replace.entry <Any-Return> "$w.next invoke"
  }
  #--
  #  Clear the selection.
  #--
  $t tag remove sel 0.0 end
  set selRange {}
  #--
  #  Check if the strings are set. Trace the string changes.
  #--
  checkSearch $w $flag dummy dummy dummy
  trace variable searchString w "checkSearch $w $flag"
  set oldFocus [focus]
  #--
  #  Get focus.
  #--
  focus $w
  #--
  #  Wait for the window vanishes.
  #--
  tkwait window $w
  #--
  #  Free the trace.
  #--
  trace vdelete searchString w "checkSearch $w $flag"
  #--
  #  Return the focus and exit.
  #--
  focus $oldFocus
}

#-- 
#  Check if the target strings are specified.
# 	w	: path of the search/replace window
# 	flag	: flag to allow replace.
# 	name1	: dummy.
#	name2 	: dummy.
#	op	: dummy.
#--
proc checkSearch {w flag name1 name2 op} {
  global searchString
  if {$searchString == ""} {
    $w.next configure -state disabled
    if $flag {
      $w.replace configure -state disabled
      $w.all configure -state disabled
    }
  } else { 
    $w.next configure -state normal
    if $flag {
      $w.replace configure -state normal
      $w.all configure -state normal
    }
  }
}

#-- 
#  Toggle the "Search" and "Replace" windows.
# 	name1	: dummy.
#	name2 	: dummy.
#	op	: dummy.
#--
proc ChangeCmd {name1 name2 op} {
  global direction searchCmd replaceCmd
  if {$direction == "up"} {
    set searchCmd "searchUp"
    set replaceCmd "replaceUp"
  } else {
    set searchCmd "searchDown"
    set replaceCmd "replaceDown"
  }
}

#-- 
#  Go to a given line.
#	t	: text widget path.
#	line	: line number.
#--
proc GoToLine {t line} {
  global lineno
  scan [$t index end] %d nolines
  if { $line > $nolines } {
     $t mark set insert $nolines.end
  } else {
     $t mark set insert $line.0
  }
  set lineno [GetLineNo $t]
  $t see insert
  focus $t
}
#-- 
#  Get line number.
#	t	: text widget path.
#--
proc GetLineNo {t} {
  global lineno
  scan [$t index insert] %d.%d line char
  set lineno $line
  return $line
}

#--
#  Chenge commands when the search direction changed.  
#--
trace variable direction w {ChangeCmd}
