#--------------- 
# edslfile.tcl 
#---------------
#
#--
#  Select a file from a listbox.
# 	w	: top level widget path name. 
# 	title	: title. 
# 	flag	: flag to allow a new file.
#--
proc SelectFile {w title flag} {
  global selected
  #--
  #  Setup the widgets.
  #--
  toplevel $w
  wm title $w $title
  frame $w.frame
  label $w.frame.dir -text "Directory"
  label $w.frame.pwd
  label $w.frame.file -text "File"
  entry $w.frame.fileName -relief sunken -bd 2
  listbox $w.frame.fileList -width 30 -height 10 -relief sunken \
      -yscrollcommand "$w.frame.scrollbar set" -selectmode single
  scrollbar $w.frame.scrollbar -orient vertical -relief sunken \
      -command "$w.frame.fileList yview"
  button $w.select -width 10 -text "Select" -command "selectFile $w.frame $flag"
  button $w.cansel -width 10 -text "Cancel" -command {set selected {}}
  #--
  #  Pack the widgets.
  #--
  pack $w.frame.dir -anchor w
  pack $w.frame.pwd -anchor w
  pack $w.frame.file -anchor w
  pack $w.frame.fileName -fill x 
  pack $w.frame.scrollbar -side right -fill y
  pack $w.frame.fileList -side left -fill x -expand 1
  pack $w.frame -side left -padx 10 -pady 10
  pack $w.select $w.cansel -padx 10 -pady 10
  #--
  #  Button and key bindings.
  #--
  bind $w.frame.fileList <ButtonRelease-1> "showEntry $w.frame"  
  bind $w.frame.fileList <Double-ButtonPress-1> "selectFile $w.frame $flag"
  bind $w.frame.fileName <Any-Return> "selectFile $w.frame $flag" 
  bind $w.frame.fileName <Left> {
      %W select clear
      %W icursor [expr [%W index insert] -1]
      tk_entrySeeCaret %W
  }
  bind $w.frame.fileName <Right> {
      %W select clear
      %W icursor [expr [%W index insert] +1]
      tk_entrySeeCaret %W
  }
  #--  
  #  List the files.
  #--
  setFileList $w.frame 
  #--
  #  Window manager setting.
  #--
  wm withdraw $w
  update idletasks
  set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
      - [winfo vrootx [winfo parent $w]]]
  set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
      - [winfo vrooty [winfo parent $w]]]
  wm geom $w +$x+$y
  wm deiconify $w
  set oldFocus [focus]
  grab $w
  focus $w.frame.fileName
  wm protocol $w WM_DELETE_WINDOW {set selected {}}
  #--
  #  Wait for the file to be selected.
  #--
  tkwait variable selected
  #--
  #  A file selected. Return to the caller.
  #--
  destroy $w
  focus $oldFocus
  return $selected
}

#--
#  Update the file list.
# 	f	: widget path of the frame containing the listbox.
#--
proc setFileList f {
  set pwd [pwd]
  #--
  #  If the file name is too long, trim off the middle part.
  #--
  if {[string length $pwd] > 30} {
     set list [split [string trim $pwd "/"] "/"]
     set len [llength $list]
     set prev ""
     for {set i $len} {1} {incr i -1} {
       set elem [lindex $list [expr $i - 1]]
       set cur "$elem/$prev"
       if {[string length $cur] > 25} {
         if {$i == $len} {set prev $elem} 
         set pwd "/.../$prev"
         break;
       }
       set prev $cur
     }
  }
  #--
  #  Update the pwd label.
  #--
  $f.pwd configure -text $pwd
  #--
  #  Update the file list.
  #--
  $f.fileList delete 0 end
  $f.fileName delete 0 end
  if [catch {set files [glob .*]}] {
    set files {}
  }
  catch {set files [concat $files [lsort [glob *]]]} 
  foreach file $files {
    if [file isdirectory $file] {set file $file/}
    $f.fileList insert end $file
  }
} 


#--
#  Decide the file name.
# 	f	: widget path of the frame containing the listbox.
# 	flag	: flag to allow a new file.
#--
proc selectFile {f flag} {
  global selected
  #--
  #  Get the selected file name.
  #--
  set name [$f.fileName get] 
  if {$name  != "/"} {
    set name [string trimright $name "/"]
  }
  if [file isdirectory $name] {
    cd $name
    setFileList $f
    return 
  }
  if {[file exists $name] || $flag} {
    set selected $name
    return
  }
  $f.fileName delete 0 end
}

#--
#  Show the selected file in the entrybox.
# 	f	: widget path of the frame containing the listbox.
#--
proc showEntry f {
  set index [$f.fileList curselection]
  if {$index == ""} return
  $f.fileName delete 0 end;
  $f.fileName insert 0 [$f.fileList get $index]
}
